/*  o+o+o+o+o+o+o+o+o+o+o+o+o+o+o+o+o+o+o+o+o+o+o+o+o+o+o+o+o+o+o+o
    o+
    o+     File:         stemnt.h
    o+
    o+     Programmer:   Shaun Flisakowski
    o+     Date:         Aug 9, 1998
    o+
    o+     A high-level view of statements.
    o+
    o+o+o+o+o+o+o+o+o+o+o+o+o+o+o+o+o+o+o+o+o+o+o+o+o+o+o+o+o+o+o+o  */

#ifndef    STEMNT_H
#define    STEMNT_H

#include <stdlib.h>

#include "express.h"

// o+o+o+o+o+o+o+o+o+o+o+o+o+o+o+o+o+o+o+o+o+o+o+o+o+o+o+o+o+o+o+o
enum StatementType
{
    ST_Empty=0,              // The null statement.

    ST_Declaration,
    ST_ExpressionStemnt,

    ST_IfStemnt,
    ST_SwitchStemnt,

    ST_ForStemnt,
    ST_WhileStemnt,
    ST_DoWhileStemnt,

    ST_ContinueStemnt,
    ST_BreakStemnt,
    ST_GotoStemnt,
    ST_ReturnStemnt,

    ST_Block 
};

// o+o+o+o+o+o+o+o+o+o+o+o+o+o+o+o+o+o+o+o+o+o+o+o+o+o+o+o+o+o+o+o
class Label
{
  public:
};

// o+o+o+o+o+o+o+o+o+o+o+o+o+o+o+o+o+o+o+o+o+o+o+o+o+o+o+o+o+o+o+o

class Statement
{
  public:
    StatementType    type;
    int              level;    // The "indent" level.


    int              nlabels;  // The number of labels on this statment.
    Label          **label;    // Array of labels.

    Statement       *next;     // For making a list of statements.
};

class ExpressionStemnt : public Statement
{
  public:
    Expression    *expression;
};

// o+o+o+o+o+o+o+o+o+o+o+o+o+o+o+o+o+o+o+o+o+o+o+o+o+o+o+o+o+o+o+o
class IfStemnt : public Statement
{
  public:
    Expression    *cond;

    Statement     *thenBlk;
    Statement     *elseBlk;        // Can be NULL
};

class SwitchStemnt : public Statement
{
  public:
    Expression    *cond;
};

// o+o+o+o+o+o+o+o+o+o+o+o+o+o+o+o+o+o+o+o+o+o+o+o+o+o+o+o+o+o+o+o
class ForStemnt : public Statement
{
  public:
    Expression    *init;
    Expression    *cond;
    Expression    *incr;

    Statement     *block;
};

class WhileStemnt : public Statement
{
  public:
    Expression    *cond;
    Statement     *block;
};

class DoWhileStemnt : public Statement
{
  public:
    Expression    *cond;
    Statement     *block;
};

// o+o+o+o+o+o+o+o+o+o+o+o+o+o+o+o+o+o+o+o+o+o+o+o+o+o+o+o+o+o+o+o
class JumpStemnt : public Statement
{
  public:
};

class GotoStemnt : public JumpStemnt
{
  public:
    Label        *dest;
};

class ReturnStemnt : public JumpStemnt
{
  public:
    Expression    *result;        // Can be NULL.
};

// o+o+o+o+o+o+o+o+o+o+o+o+o+o+o+o+o+o+o+o+o+o+o+o+o+o+o+o+o+o+o+o
class Block : public Statement
{
  public:
    Statement     *head;    // List of statements in this block.
    Statement     *tail;

    void Add(Statement *stemnt);
};

// o+o+o+o+o+o+o+o+o+o+o+o+o+o+o+o+o+o+o+o+o+o+o+o+o+o+o+o+o+o+o+o

#endif  /* STEMNT_H */

